package de.cscherr.mcpht.config;

import de.cscherr.mcpht.MCPHTClient;
import me.shedaniel.autoconfig.ConfigData;
import me.shedaniel.autoconfig.annotation.Config;
import me.shedaniel.autoconfig.annotation.ConfigEntry;

import java.util.regex.Pattern;

@Config(name = "mcphtconf")
public class MCPHTConfig implements ConfigData {
    public static class NetworkLoggingScheme {
        public boolean RX = false;
        public boolean TX = false;
        public NetworkPacketVerbosity verbosity = NetworkPacketVerbosity.NAME;
        public String regex = ".*";
        public enum NetworkPacketVerbosity {
            ALL, NAME
        }
    }
    @ConfigEntry.Category("Networking")
    @ConfigEntry.Gui.CollapsibleObject
    public NetworkLoggingScheme networkLogging = new NetworkLoggingScheme();
    @ConfigEntry.Category("Rendering")
    @ConfigEntry.Gui.CollapsibleObject
    public XRayScheme xRay = new XRayScheme();
    // TODO: toggle with hotkey
    public static class XRayScheme {
        public boolean enable = false;
        public boolean overwriteBrightness;
    }
    @ConfigEntry.Category("Movement")
    @ConfigEntry.Gui.CollapsibleObject
    public FlyScheme fly = new FlyScheme();
    // TODO: toggle with hotkey
    public static class FlyScheme {
        public boolean enable = false;
    }
    public void onSave() {
        MCPHTClient.LOGGER.info("saving and processing configs");
        MCPHTClient.networkPacketFilter = compileFilter();

        MCPHTClient.MC.worldRenderer.reload();
        logSettings();
    }
    public void onLoad() {
        MCPHTClient.LOGGER.info("loading configs");
        MCPHTClient.networkPacketFilter = compileFilter();

        if (MCPHTClient.MC.worldRenderer != null) {
            MCPHTClient.MC.worldRenderer.reload();
        }
        logSettings();
    }
    public void logSettings() {
    // TODO: autogenerate this from Hack classes
        MCPHTClient.LOGGER.info(String.format("""
                        Networking
                            NetworkLogging
                            	RX: %b
                            	TX: %b
                            	verbosity: %s
                            	regex: '%s'
                            	filter: %s
                        Rendering
                            XRay:
                                enabled: %b
                            XRay:
                                enabled: %b
                        """,
                MCPHTClient.CONFIG.networkLogging.RX,
                MCPHTClient.CONFIG.networkLogging.TX,
                MCPHTClient.CONFIG.networkLogging.verbosity,
                MCPHTClient.CONFIG.networkLogging.regex,
                MCPHTClient.networkPacketFilter,

                MCPHTClient.CONFIG.xRay.enable,
                MCPHTClient.CONFIG.fly.enable
        ));
    }
    private Pattern compileFilter() {
        try {
            return Pattern.compile(this.networkLogging.regex);
        } catch (Exception e) {
            MCPHTClient.LOGGER.warn(String.format("Invalid regex: %s", this.networkLogging.regex));
            return Pattern.compile(".*"); // just accept all on error
        }
    }
}
